/*
   xdm_screen.cc
*/

#include <stdio.h>
#include <string.h>

#include "const.h"
#include "xdm_screen.h"
#include "xdm_timer.h"


ScreenWidget::ScreenWidget( QWidget *parent = 0, const char *name = 0)
: QWidget( parent, name)
{
    TimerObject  *timer;
    
    
    setMinimumSize( 256, 256);
    setMaximumSize( 256, 256);
    
    setCaption( "dynamate 0.1");
    
    released = TRUE;
    
    screen = new DynaPixels( 256, 256);
    screen->dynscr = this;
    timer = new TimerObject();
    timer->screen = screen;
    
    dynfile = new QFileDialog( 0, 0, FALSE);
    dynfile->setCaption( "select a level");
    connect( dynfile, SIGNAL( fileHighlighted( const char*)),
        SLOT( loadLevel( const char*)));
}

void ScreenWidget::paintEvent( QPaintEvent *evt)
{
/*    int  left = pos2index( evt->rect().left);
    int  top = pos2index( evt->rect().top );*/
/*    int  width = evt->rect().width, height = evt->rect().height;*/
    
    
    bitBlt( this, 0, 0, screen);
}

void ScreenWidget::mousePressEvent( QMouseEvent *evt)
{
    if ( evt->button() == RightButton )
    {
        dynfile->show();
        return;
    }
    
    if ( evt->button() == LeftButton && released && screen->posx != -1 &&
        screen->dirx == 0 && screen->diry == 0 )
    {
        released = FALSE;
        screen->posx = evt->x() / 16;
        screen->posy = evt->y() / 16;
    }
}

void ScreenWidget::mouseReleaseEvent( QMouseEvent *evt)
{
    int  x, y;
    
    
    if ( evt->button() == LeftButton && screen->posx != -1 &&
        screen->dirx == 0 && screen->diry == 0 )
    {
        released = true;
        x = evt->x() / 16 - screen->posx;
        y = evt->y() / 16 - screen->posy;
        
        if ( x == 0 && y == 0 ) return;
        if ( !(screen->leveldata[ screen->posx ][ screen->posy ] & 128) ) return;
        if ( screen->dirx != 0 || screen-> diry != 0 ) return;
        
        if ( x < 0 )
        {
            screen->dirx = -1;
            x = -x;
        }
        else screen->dirx = 1;
        
        if ( y < 0 )
        {
            screen->diry = -1;
            y = -y;
        }
        else screen->diry = 1;
        
        if ( x > y ) screen->diry = 0;
        else screen->dirx = 0;
        
        screen->moves++;
    }
}

void ScreenWidget::loadLevel( const char *filename)
{
    FILE  *level;
    int   x, y;
    
    
    strcpy( screen->curlevel, filename);
    
    level = fopen( filename, "rb");
    if ( level == 0L ) return;
    
    screen->stonesleft = 0;
    for ( y = 0; y < 16; y++ ) for ( x = 0; x < 16; x++ )
    {
        screen->leveldata[ x ][ y ] = fgetc( level);
        switch( screen->leveldata[ x ][ y ] & 127 )
        {
             case S_BLACK:
             case S_YELLOW:
             case S_RED:
             case S_GREEN:
             case S_BLUE:
                 screen->stonesleft++;
                 break;
        }
    }
    
    if ( !feof( level) ) screen->best = fgetc( level);
    else screen->best = 250;
    
    fclose( level);
    
    screen->gluesleft = 0;
    screen->moves = 0;
    screen->drawLevel();
}
